@@ -160,6 +160,11 @@ void env_defaults(void) {
 	// set the window title
 	if (!arg_quiet)
 		printf("\033]0;firejail %s\007", cfg.window_title);
+
+	// pass --quiet as an environment variable, in case the command calls further firejailed commands
+	if (arg_quiet)
+		setenv("FIREJAIL_QUIET", "yes", 1);
+
 	fflush(0);
 }
 
