@@ -143,10 +143,18 @@ static int check_appimage(void) {
 	return arg_appimage != 0;
 }
 
+static int check_netoptions(void) {
+	return (arg_nonetwork || any_bridge_configured());
+}
+
 static int check_nodbus(void) {
 	return arg_nodbus != 0;
 }
 
+static int check_x11(void) {
+	return (arg_x11_block || arg_x11_xorg || getenv("FIREJAIL_X11"));
+}
+
 static int check_disable_u2f(void) {
 	return checkcfg(CFG_BROWSER_DISABLE_U2F) != 0;
 }
@@ -157,7 +165,9 @@ static int check_allow_drm(void) {
 
 Cond conditionals[] = {
 	{"HAS_APPIMAGE", check_appimage},
+	{"HAS_NET", check_netoptions},
 	{"HAS_NODBUS", check_nodbus},
+	{"HAS_X11", check_x11},
 	{"BROWSER_DISABLE_U2F", check_disable_u2f},
 	{"BROWSER_ALLOW_DRM", check_allow_drm},
 	{ NULL, NULL }
@@ -869,8 +879,8 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		else if (cfg.dns4 == NULL)
 			cfg.dns4 = dns;
 		else {
-			fprintf(stderr, "Error: up to 4 DNS servers can be specified\n");
-			exit(1);
+			fwarning("Warning: up to 4 DNS servers can be specified, %s ignored\n", dns);
+			free(dns);
 		}
 		return 0;
 	}
@@ -941,6 +951,11 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 
+	if (strcmp(ptr, "allow-debuggers") == 0) {
+		arg_allow_debuggers = 1;
+		return 0;
+	}
+
 	if (strcmp(ptr, "x11 none") == 0) {
 		arg_x11_block = 1;
 		return 0;
