@@ -268,6 +268,7 @@ static int monitor_application(pid_t app_pid) {
 	if (cfg.timeout) {
 		options = WNOHANG;
 		timeout = cfg.timeout;
+		sleep(1);
 	}
 
 	int status = 0;
@@ -302,8 +303,11 @@ static int monitor_application(pid_t app_pid) {
 			// handle --timeout
 			if (options) {
 				if (--timeout == 0)  {
+					// SIGTERM might fail if the process ignores it (SIG_IGN)
+					// we give it 100ms to close properly and after that we SIGKILL it
 					kill(-1, SIGTERM);
-					sleep(1);
+					usleep(100000);
+					kill(-1, SIGKILL);
 					flush_stdin();
 					_exit(1);
 				}
@@ -574,7 +578,7 @@ static void enforce_filters(void) {
 	force_nonewprivs = 1;
 
 	// disable all capabilities
-	fmessage("\n**     Warning: dropping all Linux capabilities     **\n");
+	fmessage("\n**     Warning: dropping all Linux capabilities     **\n\n");
 	arg_caps_drop_all = 1;
 
 	// drop all supplementary groups; /etc/group file inside chroot
@@ -1106,7 +1110,7 @@ int sandbox(void* sandbox_arg) {
 		(void) rv;
 	}
 	// make seccomp filters read-only
-	fs_rdonly(RUN_SECCOMP_DIR);
+	fs_remount(RUN_SECCOMP_DIR, MOUNT_READONLY, 0);
 #endif
 
 	// set capabilities
