@@ -26,6 +26,7 @@
 #include <sys/stat.h>
 
 pid_t skip_process = 0;
+int arg_debug = 0;
 static int arg_route = 0;
 static int arg_arp = 0;
 static int arg_tree = 0;
@@ -83,7 +84,9 @@ int find_child(int id) {
 			return i;
 	}
 
-	return -1;
+	// if a second child is not found, return the first child pid
+	// this happens for processes sandboxed with --join
+	return first_child;
 }
 
 // sleep and wait for a key to be pressed
@@ -140,7 +143,8 @@ int main(int argc, char **argv) {
 			printf("firemon version %s\n\n", VERSION);
 			return 0;
 		}
-
+		else if (strcmp(argv[i], "--debug") == 0)
+			arg_debug = 1;
 		// options without a pid argument
 		else if (strcmp(argv[i], "--top") == 0)
 			arg_top = 1;
